import Trace::*;
import Vector::*;
import PathTypes::*;

function FullPath genEmptyPaths (Integer a) ;
   FullPath empt = FullPath {voice: fromInteger(a), startAddr: 0, route: emptyCore, outVol: 0};
   return (empt);
endfunction

interface RoutingTable;
   method FullPath getPath(Integer voiceNo);
   method Vector#(`MAX_VOICES, Volume) getMixerScalars();
endinterface

module mkRoutingTable(RoutingTable);
   Vector#(`MAX_VOICES, FullPath) routeTable = genWith(genEmptyPaths);

   function CorePath updateVoice0(Integer len, Vector#(len, PathId) cores, CorePath inPath);
      CorePath outPath = inPath;
      for(Integer i = 0; i < len; i = i+1)
	inPath[i] = cores[i];
      return outPath;
   endfunction

   //demonstrate two ways of building the routeTable
   routeTable[0].startAddr = 0;  //where is this really going to come from?
   routeTable[0].route[0] = 3;
   routeTable[0].route[1] = mixerId;
   routeTable[0].outVol = 200;
   //rest are already initialized toinvalid

   // or if you just want to update a straight list, this longer template works.
   function CorePath createVoice1Route();
      CorePath outPath = emptyCore;
      
      Integer len = 3;
      PathId route[len] = {1, 2, mixerId}; //route to update
 
      for(Integer i = 0; i < len; i = i+1)
	 outPath[i] = route[i];
      
      return outPath;
   endfunction
   routeTable[1].route = createVoice1Route;
   routeTable[1].startAddr = 0;   
   routeTable[1].outVol = 200;

   //remaining voices are all initialized to empty.
   
   method FullPath getPath(Integer a);      
      return routeTable[a];
   endmethod

   method Vector#(`MAX_VOICES, Volume) getMixerScalars();
      Vector#(`MAX_VOICES, Volume) scalars = newVector();
      for(Integer i = 0; i < `MAX_VOICES; i = i+1)
	 scalars[i] = routeTable[i].outVol;
      return scalars;
   endmethod
   
endmodule
 
